local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local LayoutAttrib = BaikCore.LayoutAttrib
local Log = BaikCore.Log
local Option = BaikCore.Option

-- Add Module
local CastBar = BaikUnitFrame:NewModule("CastBar", "AceHook-3.0")

-- Constants
local CASTBAR_DB_NAME = "castbar_enable"
local CASTBAR_DEFAULT = true

local CASTBAR_LAYOUT = {
    ["CastingBarFrame"] = LayoutAttrib:Create(119, 12, "TOPLEFT", 106, -52),
    ["TargetFrameSpellBar"] = LayoutAttrib:Create(119, 12, "TOPRIGHT", -106, -52),
    ["FocusFrameSpellBar"] = LayoutAttrib:Create(119, 12, "TOPRIGHT", -106, -52),
}

local CASTBAR_TEXT = {
    ["CastingBarFrame"] = LayoutAttrib:Create(119, nil, "TOP", 0, 2),
    ["TargetFrameSpellBar"] = LayoutAttrib:Create(119, nil, "TOP", 0, 2),
    ["FocusFrameSpellBar"] = LayoutAttrib:Create(119, nil, "TOP", 0, 2),
}

local CASTBAR_UNITFRAME = {
    ["CastingBarFrame"] = PlayerFrame,
    ["TargetFrameSpellBar"] = TargetFrame,
    ["FocusFrameSpellBar"] = FocusFrame,
}

-- Private Methods
local function LoadCastbar(castbar)
    Log:v("Loading castbar")
    -- Ensure mappings exist
    local name = castbar:GetName()
    local barAttrib = CASTBAR_LAYOUT[name]
    local textAttrib = CASTBAR_TEXT[name]
    local unitFrame = CASTBAR_UNITFRAME[name]
    Assert:NotNull(barAttrib)
    Assert:NotNull(textAttrib)
    Assert:NotNull(unitFrame)

    -- Framelevel
    castbar:SetToplevel(false)
    castbar:SetFrameStrata("LOW")
    castbar:SetParent(unitFrame)

    -- Hide elements
    castbar.Icon:Hide()
    castbar.Border:Hide()
    castbar.BorderShield:Hide()

    -- Sync Layout
    barAttrib:SyncRegion(castbar, true)
    textAttrib:SyncRegion(castbar.Text, true)
end

local function LoadCastbarFinal(castbar)
    Log:v("Loading castbar final")

    -- Ensure mappings exist
    local name = castbar:GetName()
    local barAttrib = CASTBAR_LAYOUT[name]
    local textAttrib = CASTBAR_TEXT[name]
    local unitFrame = CASTBAR_UNITFRAME[name]
    Assert:NotNull(barAttrib)
    Assert:NotNull(textAttrib)
    Assert:NotNull(unitFrame)

    -- Framelevel
    castbar:SetToplevel(false)
    castbar:SetFrameStrata("LOW")
    castbar:SetParent(unitFrame)

    -- Hide elements
    castbar.Icon:Hide()
    castbar.Border:Hide()
    castbar.BorderShield:Hide()

    -- Sync Layout
    barAttrib:SyncRegion(castbar, true)
    textAttrib:SyncRegion(castbar.Text, true)

    -- Set Noop
    Hook:EnsureNoOp(CastBar, castbar, "SetParent", true)
    Hook:EnsureNoOp(CastBar, castbar, "SetPoint", true)
end

-- PostHook Methods
local function PostHook_PlayerFrame_AttachCastBar(...)
    LoadCastbar(CastingBarFrame)
end

local function PostHook_PlayerFrame_DetachCastBar(...)
    Log:e("Disable attached castbar in options to detach castbar")
    PLAYER_FRAME_CASTBARS_SHOWN = true
    PlayerFrame_AttachCastBar()
end

local HOOK_TABLE = {
    ["PlayerFrame_AttachCastBar"] = PostHook_PlayerFrame_AttachCastBar,
    ["PlayerFrame_DetachCastBar"] = PostHook_PlayerFrame_DetachCastBar,
}

local function HookCastBar(value)
    for method, handle in pairs(HOOK_TABLE) do
        if value then
            Hook:EnsureSecureHook(CastBar, method, handle)
        else
            Hook:EnsureUnhook(CastBar, method, handle)
        end
    end

    if value then
        PlayerFrame_AttachCastBar()
        LoadCastbarFinal(TargetFrameSpellBar)
        LoadCastbarFinal(FocusFrameSpellBar)
    end
end

local function GetOpt()
    return {
        name = "Castbar",
        type = "group",
        args = {
            attach_castbar = {
                name = "Attach UnitFrame",
                type = "toggle",
                get = function(...)
                    return Database:Get(CASTBAR_DB_NAME)
                end,
                set = function(info, value, ...)
                    Database:Set(CASTBAR_DB_NAME, value)
                    HookCastBar(value)
                end
            }
        }
    }
end

-- Ace3 Callbacks
function CastBar:OnInitialize()
    Log:v("CastBar_OnInitialize")
    Database:InitIfEmpty(CASTBAR_DB_NAME, CASTBAR_DEFAULT)
    Option:AddOption(BaikUnitFrame.OPTION_NAME, "castbar", GetOpt())
end

function CastBar:OnEnable()
    Log:v("CastBar_OnEnable")
    HookCastBar(Database:Get(CASTBAR_DB_NAME))
end

function CastBar:OnDisable()
    Log:v("CastBar_OnDisable")
end
