local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikUnitFrame = BaikCore.BaikUnitFrame
local Database = BaikCore.Database
local Hook = BaikCore.Hook
local Log = BaikCore.Log
local Option = BaikCore.Option

-- Add Module
local HealthBar = BaikUnitFrame:NewModule("HealthBar", "AceHook-3.0")

-- Constants
local CLASS_COLOR_DB = "health_class_color"
local COLOR_DEFAULT = true
local CLASS_HOOKS = nil

-- Private Methods
local function SetClassColor(statusbar, unit)
    if (unit == statusbar.unit
        and not statusbar.lockColor
        and not statusbar.disconnected
        and UnitIsPlayer(unit)) then
        local class = select(2, UnitClass(unit))
        local useClassColor = RAID_CLASS_COLORS[class]
        if useClassColor then
            statusbar:SetStatusBarColor(useClassColor.r, useClassColor.g, useClassColor.b)
        end
    end
end

-- Hook Methods
local function PostHook_UnitFrameHealthBar_Update(statusbar, unit, ...)
    SetClassColor(statusbar, unit)
end

local function PostHook_HeathBar_OnValueChanged(statusbar, ...)
    SetClassColor(statusbar, statusbar.unit)
end


local function ClassHookTable()
    if not CLASS_HOOKS then
        CLASS_HOOKS = {
            ["UnitFrameHealthBar_Update"] = PostHook_UnitFrameHealthBar_Update,
            ["HealthBar_OnValueChanged"] = PostHook_HeathBar_OnValueChanged,
        }
    end

    return CLASS_HOOKS
end

local function HookClassColor(value)
    local hookTable = ClassHookTable()

    for method, handle in pairs(hookTable) do
        if value then
            Hook:EnsureSecureHook(HealthBar, method, handle)
        else
            Hook:EnsureUnhook(HealthBar, method, handle)
        end
    end
end

local function GetOpt()
    return {
        name = "HeathBar",
        type = "group",
        args = {
            class_color = {
                name = "Class Color",
                type = "toggle",
                get = function(...)
                    return Database:Get(CLASS_COLOR_DB)
                end,
                set = function(info, value, ...)
                    Database:Set(CLASS_COLOR_DB, value)
                    HookClassColor(value)
                end
            }
        }
    }
end

-- Ace3 Callbacks
function HealthBar:OnInitialize()
    Log:v("HealthBar_OnInitialize")
    -- Initialize Database``
    Database:InitIfEmpty(CLASS_COLOR_DB, COLOR_DEFAULT)

    -- Add Options
    Option:AddOption(BaikUnitFrame.OPTION_NAME, "class_color", GetOpt())
end

function HealthBar:OnEnable()
    Log:v("HealthBar_OnEnable")
    -- Kick off hooks
    HookClassColor(Database:Get(CLASS_COLOR_DB))
end

function HealthBar:OnDisable()
    Log:v("HealthBar_OnDisable")
    self:UnhookAll()
end
